@@ -227,7 +227,7 @@ For the DISPLAY to match a FRAME, the REQ property of the frame must
 match one of the ITEM.  The following REQ are defined:
 
 `type' (the value of `window-system')
-  Should be one of `x' or `tty'.
+  Should be one of `x', `mswindows', or `tty'.
 
 `class' (the frame's color support)
   Should be one of `color', `grayscale', or `mono'.
