@@ -22,57 +22,74 @@ Boston, MA 02111-1307, USA.  */
 
 /* The purpose of this file is so that there is at least one actual
    definition of each inline function.  This is needed under GCC.  The
-   reason is that under GCC we declare our inline functions `extern
-   inline', which causes the inlined version to get used only for
+   reason is that under GCC we declare our inline functions `inline
+   extern', which causes the inlined version to get used only for
    inlining, and in other cases to generate an external reference to
    the function.  This is more efficient than declaring our inline
-   functions `static inline', which (in many cases) would cause a separate
+   functions `inline static', which (in many cases) would cause a separate
    version of the function to get inserted into every source file that
-   included the corresponding header file.
+   included the corresponding header file.  See internals.texi.
 
    Some compilers that recognize `inline' may not do the same
-   `extern inline' business, so on those we just do `static inline'.
+   `inline extern' business, so on those we just do `inline static'.
    */
 
-#define DONT_EXTERN_INLINE_FUNCTIONS
+/* Note to maintainers: This file contains a list of all header files
+   that use the INLINE macro, either directly, or by using DECLARE_LRECORD.
+   i.e. the output of ``grep -l -w 'DECLARE_LRECORD|INLINE_HEADER' *.h'' */
+
+#define DONT_EXTERN_INLINE_HEADER_FUNCTIONS
 
 #include <config.h>
 #include "lisp.h"
 #include "buffer.h"
 #include "bytecode.h"
+#include "chartab.h"
 #include "console.h"
 #include "device.h"
-#include "events.h"
 #include "elhash.h"
+#include "events.h"
 #include "extents.h"
 #include "faces.h"
 #include "frame.h"
 #include "glyphs.h"
+#include "gui.h"
 #include "keymap.h"
 #include "lstream.h"
 #include "objects.h"
 #include "opaque.h"
 #include "process.h"
+#include "rangetab.h"
 #include "specifier.h"
 #include "syntax.h"
-#include "toolbar.h"
 #include "window.h"
 
+#ifdef HAVE_LDAP
+#include "eldap.h"
+#endif
+
+#ifdef HAVE_POSTGRESQL
+#include <libpq-fe.h>
+#include "postgresql.h"
+#endif
+
+#ifdef HAVE_TOOLBARS
+#include "toolbar.h"
+#endif
+
 #ifdef HAVE_DATABASE
 #include "database.h"
 #endif
 
 #ifdef HAVE_X_WINDOWS
-#include "console-x.h"
 #include "glyphs-x.h"
 #include "gui-x.h"
 #endif
 
-#ifdef MULE
-#include "mule-coding.h"
+#ifdef FILE_CODING
+#include "file-coding.h"
 #endif
 
 #ifdef TOOLTALK
-#include TT_C_H_PATH
 #include "tooltalk.h"
 #endif
