@@ -158,7 +158,7 @@ invalidate_line_number_cache (struct buffer *b, Bufpos pos)
    This will do nothing if the cache is uninitialized.  */
 void
 insert_invalidate_line_number_cache (struct buffer *b, Bufpos pos,
-				     CONST Bufbyte *nonreloc, Bytecount length)
+				     const Bufbyte *nonreloc, Bytecount length)
 {
   if (NILP (b->text->line_number_cache))
     return;
@@ -293,7 +293,7 @@ buffer_line_number (struct buffer *b, Bufpos pos, int cachep)
       get_nearest_line_number (b, &beg, pos, &cached_lines);
     }
 
-  scan_buffer (b, '\n', beg, pos, pos > beg ? EMACS_INT_MAX : -EMACS_INT_MAX,
+  scan_buffer (b, '\n', beg, pos, pos > beg ? EMACS_INT_MAX : EMACS_INT_MIN,
 	       &shortage, 0);
 
   line = EMACS_INT_MAX - shortage;
