@@ -24,11 +24,6 @@ Boston, MA 02111-1307, USA.  */
    operating system this machine is likely to run.
    USUAL-OPSYS="usg5-2"  */
 
-/* Define WORD_MACHINE if addresses and such have
- * to be corrected before they can be used as byte counts.  */
-
-#undef WORD_MACHINE
-
 /* Now define a symbol for the cpu type, if your compiler
    does not define it automatically:
    vax, m68000, ns16000 are the ones defined so far.  */
@@ -40,10 +35,6 @@ Boston, MA 02111-1307, USA.  */
 /* This flag is used only in alloca.s.  */
 #define WICAT
 
-/* XINT must explicitly sign-extend */
-
-#define EXPLICIT_SIGN_EXTEND
-
 /* Data type of load average, as read out of kmem.  */
 
 #undef LOAD_AVE_TYPE
@@ -106,10 +97,6 @@ Boston, MA 02111-1307, USA.  */
 
 #define LD_SWITCH_MACHINE "-e __start -N"
 
-/* Sigh...cannot define this for WICAT cuz 0 length memcpy blows chunks */
-
-#define BROKEN_BCOPY
-
 /*
  * Define optimflags if you want to optimize.
  *	- Set to null string for pre-4.2 C compiler
