@@ -2,7 +2,7 @@
    Copyright (C) 1985, 1986, 1987, 1988, 1992, 1993, 1994, 1995
    Free Software Foundation, Inc.
    Copyright (C) 1995 Sun Microsystems, Inc.
-   Copyright (C) 1995, 1996 Ben Wing.
+   Copyright (C) 1995, 1996, 2000 Ben Wing.
 
 This file is part of XEmacs.
 
@@ -26,15 +26,15 @@ Boston, MA 02111-1307, USA.  */
 #include <config.h>
 #include "lisp.h"
 
+#include "console-msw.h"
 #include "hash.h"
 #include "lstream.h"
 #include "process.h"
 #include "procimpl.h"
 #include "sysdep.h"
 
-#ifndef __MINGW32__
 #include <shellapi.h>
-#else
+#ifdef __MINGW32__
 #include <errno.h>
 #endif
 #include <signal.h>
@@ -52,9 +52,22 @@ Lisp_Object Qnt_quote_process_args;
 struct nt_process_data
 {
   HANDLE h_process;
+  DWORD dwProcessId;
+  HWND hwnd; /* console window */
   int need_enable_child_signals;
 };
 
+/* Control whether create_child causes the process to inherit Emacs'
+   console window, or be given a new one of its own.  The default is
+   nil, to allow multiple DOS programs to run on Win95.  Having separate
+   consoles also allows Emacs to cleanly terminate process groups.  */
+Lisp_Object Vmswindows_start_process_share_console;
+
+/* Control whether create_child cause the process to inherit Emacs'
+   error mode setting.  The default is t, to minimize the possibility of
+   subprocesses blocking when accessing unmounted drives.  */
+Lisp_Object Vmswindows_start_process_inherit_error_mode;
+
 #define NT_DATA(p) ((struct nt_process_data*)((p)->process_data))
 
 /*-----------------------------------------------------------------------*/
@@ -68,6 +81,21 @@ get_nt_process_handle (Lisp_Process *p)
 {
   return (NT_DATA (p)->h_process);
 }
+
+static struct Lisp_Process *
+find_process_from_pid (DWORD pid)
+{
+  Lisp_Object tail, proc;
+
+  for (tail = Vprocess_list; CONSP (tail); tail = XCDR (tail))
+    {
+      proc = XCAR (tail);
+      if (NT_DATA (XPROCESS (proc))->dwProcessId == pid)
+	return XPROCESS (proc);
+    }
+  return 0;
+}
+
 
 /*-----------------------------------------------------------------------*/
 /* Running remote threads. See Microsoft Systems Journal 1994 Number 5	 */
@@ -224,6 +252,8 @@ run_in_other_process (HANDLE h_process,
 /* Sending signals							 */
 /*-----------------------------------------------------------------------*/
 
+/* ---------------------------- the NT way ------------------------------- */
+
 /*
  * We handle the following signals:
  *
@@ -287,19 +317,34 @@ sig_enable_proc (sig_enable_data* data)
  * Return nonzero if successful.
  */
 
-/* This code assigns a return value of GetProcAddress to function pointers
-   of many different types. Instead of heavy obscure casts, we just disable
-   warnings about assignments to different function pointer types. */
-#pragma warning (disable : 4113)
-
 static int
-send_signal (HANDLE h_process, int signo)
+send_signal_the_nt_way (struct nt_process_data *cp, int pid, int signo)
 {
+  HANDLE h_process;
   HMODULE h_kernel = GetModuleHandle ("kernel32");
+  int close_process = 0;
   DWORD retval;
   
   assert (h_kernel != NULL);
   
+  if (cp)
+    {
+      pid = cp->dwProcessId;
+      h_process = cp->h_process;
+    }
+  else
+    {
+      close_process = 1;
+      /* Try to open the process with required privileges */
+      h_process = OpenProcess (PROCESS_CREATE_THREAD
+			       | PROCESS_QUERY_INFORMATION 
+			       | PROCESS_VM_OPERATION
+			       | PROCESS_VM_WRITE,
+			       FALSE, pid);
+      if (!h_process)
+	return 0;
+    }
+
   switch (signo)
     {
     case SIGKILL:
@@ -308,7 +353,9 @@ send_signal (HANDLE h_process, int signo)
     case SIGHUP:
       {
 	sigkill_data d;
-	d.adr_ExitProcess = GetProcAddress (h_kernel, "ExitProcess");
+
+	d.adr_ExitProcess =
+	  (void (WINAPI *) (UINT)) GetProcAddress (h_kernel, "ExitProcess");
 	assert (d.adr_ExitProcess);
 	retval = run_in_other_process (h_process, 
 				       (LPTHREAD_START_ROUTINE)sigkill_proc,
@@ -319,6 +366,7 @@ send_signal (HANDLE h_process, int signo)
       {
 	sigint_data d;
 	d.adr_GenerateConsoleCtrlEvent =
+	  (BOOL (WINAPI *) (DWORD, DWORD))
 	  GetProcAddress (h_kernel, "GenerateConsoleCtrlEvent");
 	assert (d.adr_GenerateConsoleCtrlEvent);
 	d.event = CTRL_C_EVENT;
@@ -331,6 +379,8 @@ send_signal (HANDLE h_process, int signo)
       assert (0);
     }
 
+  if (close_process)
+    CloseHandle (h_process);
   return (int)retval > 0 ? 1 : 0;
 }
 
@@ -345,6 +395,7 @@ enable_child_signals (HANDLE h_process)
   
   assert (h_kernel != NULL);
   d.adr_SetConsoleCtrlHandler =
+    (BOOL (WINAPI *) (LPVOID, BOOL))
     GetProcAddress (h_kernel, "SetConsoleCtrlHandler");
   assert (d.adr_SetConsoleCtrlHandler);
   run_in_other_process (h_process, (LPTHREAD_START_ROUTINE)sig_enable_proc,
@@ -353,6 +404,214 @@ enable_child_signals (HANDLE h_process)
   
 #pragma warning (default : 4113)
 
+/* ---------------------------- the 95 way ------------------------------- */
+
+static BOOL CALLBACK
+find_child_console (HWND hwnd, struct nt_process_data *cp)
+{
+  DWORD thread_id;
+  DWORD process_id;
+
+  thread_id = GetWindowThreadProcessId (hwnd, &process_id);
+  if (process_id == cp->dwProcessId)
+    {
+      char window_class[32];
+
+      GetClassName (hwnd, window_class, sizeof (window_class));
+      if (strcmp (window_class,
+		  msw_windows9x_p ()
+		  ? "tty"
+		  : "ConsoleWindowClass") == 0)
+	{
+	  cp->hwnd = hwnd;
+	  return FALSE;
+	}
+    }
+  /* keep looking */
+  return TRUE;
+}
+
+static int
+send_signal_the_95_way (struct nt_process_data *cp, int pid, int signo)
+{
+  HANDLE h_process;
+  int close_process = 0;
+  int rc = 1;
+  
+  if (cp)
+    {
+      pid = cp->dwProcessId;
+      h_process = cp->h_process;
+
+      /* Try to locate console window for process. */
+      EnumWindows (find_child_console, (LPARAM) cp);
+    }
+  else
+    {
+      close_process = 1;
+      /* Try to open the process with required privileges */
+      h_process = OpenProcess (PROCESS_TERMINATE, FALSE, pid);
+      if (!h_process)
+	return 0;
+    }
+    
+  if (signo == SIGINT)
+    {
+      if (NILP (Vmswindows_start_process_share_console) && cp && cp->hwnd)
+	{
+	  BYTE control_scan_code = (BYTE) MapVirtualKey (VK_CONTROL, 0);
+	  BYTE vk_break_code = VK_CANCEL;
+	  BYTE break_scan_code = (BYTE) MapVirtualKey (vk_break_code, 0);
+	  HWND foreground_window;
+
+	  if (break_scan_code == 0)
+	    {
+	      /* Fake Ctrl-C if we can't manage Ctrl-Break. */
+	      vk_break_code = 'C';
+	      break_scan_code = (BYTE) MapVirtualKey (vk_break_code, 0);
+	    }
+
+	  foreground_window = GetForegroundWindow ();
+	  if (foreground_window)
+	    {
+              /* NT 5.0, and apparently also Windows 98, will not allow
+		 a Window to be set to foreground directly without the
+		 user's involvement. The workaround is to attach
+		 ourselves to the thread that owns the foreground
+		 window, since that is the only thread that can set the
+		 foreground window.  */
+              DWORD foreground_thread, child_thread;
+              foreground_thread =
+		GetWindowThreadProcessId (foreground_window, NULL);
+	      if (foreground_thread == GetCurrentThreadId ()
+                  || !AttachThreadInput (GetCurrentThreadId (),
+                                         foreground_thread, TRUE))
+                foreground_thread = 0;
+
+              child_thread = GetWindowThreadProcessId (cp->hwnd, NULL);
+	      if (child_thread == GetCurrentThreadId ()
+                  || !AttachThreadInput (GetCurrentThreadId (),
+                                         child_thread, TRUE))
+                child_thread = 0;
+
+              /* Set the foreground window to the child.  */
+              if (SetForegroundWindow (cp->hwnd))
+                {
+                  /* Generate keystrokes as if user had typed Ctrl-Break or
+                     Ctrl-C.  */
+                  keybd_event (VK_CONTROL, control_scan_code, 0, 0);
+                  keybd_event (vk_break_code, break_scan_code,
+		    (vk_break_code == 'C' ? 0 : KEYEVENTF_EXTENDEDKEY), 0);
+                  keybd_event (vk_break_code, break_scan_code,
+                    (vk_break_code == 'C' ? 0 : KEYEVENTF_EXTENDEDKEY)
+                    | KEYEVENTF_KEYUP, 0);
+                  keybd_event (VK_CONTROL, control_scan_code,
+                               KEYEVENTF_KEYUP, 0);
+
+                  /* Sleep for a bit to give time for Emacs frame to respond
+                     to focus change events (if Emacs was active app).  */
+                  Sleep (100);
+
+                  SetForegroundWindow (foreground_window);
+                }
+              /* Detach from the foreground and child threads now that
+                 the foreground switching is over.  */
+              if (foreground_thread)
+                AttachThreadInput (GetCurrentThreadId (),
+                                   foreground_thread, FALSE);
+              if (child_thread)
+                AttachThreadInput (GetCurrentThreadId (),
+                                   child_thread, FALSE);
+            }
+        }
+      /* Ctrl-Break is NT equivalent of SIGINT.  */
+      else if (!GenerateConsoleCtrlEvent (CTRL_BREAK_EVENT, pid))
+        {
+#if 0 /* FSF Emacs */
+	  DebPrint (("sys_kill.GenerateConsoleCtrlEvent return %d "
+		     "for pid %lu\n", GetLastError (), pid));
+	  errno = EINVAL;
+#endif
+	  rc = 0;
+	}
+    }
+  else
+    {
+      if (NILP (Vmswindows_start_process_share_console) && cp && cp->hwnd)
+	{
+#if 1
+	  if (msw_windows9x_p ())
+	    {
+/*
+   Another possibility is to try terminating the VDM out-right by
+   calling the Shell VxD (id 0x17) V86 interface, function #4
+   "SHELL_Destroy_VM", ie.
+
+     mov edx,4
+     mov ebx,vm_handle
+     call shellapi
+
+   First need to determine the current VM handle, and then arrange for
+   the shellapi call to be made from the system vm (by using
+   Switch_VM_and_callback).
+
+   Could try to invoke DestroyVM through CallVxD.
+
+*/
+#if 0
+	      /* On Win95, posting WM_QUIT causes the 16-bit subsystem
+		 to hang when cmdproxy is used in conjunction with
+		 command.com for an interactive shell.  Posting
+		 WM_CLOSE pops up a dialog that, when Yes is selected,
+		 does the same thing.  TerminateProcess is also less
+		 than ideal in that subprocesses tend to stick around
+		 until the machine is shutdown, but at least it
+		 doesn't freeze the 16-bit subsystem.  */
+	      PostMessage (cp->hwnd, WM_QUIT, 0xff, 0);
+#endif
+	      if (!TerminateProcess (h_process, 0xff))
+		{
+#if 0 /* FSF Emacs */
+		  DebPrint (("sys_kill.TerminateProcess returned %d "
+			     "for pid %lu\n", GetLastError (), pid));
+		  errno = EINVAL;
+#endif
+		  rc = 0;
+		}
+	    }
+	  else
+#endif
+	    PostMessage (cp->hwnd, WM_CLOSE, 0, 0);
+	}
+      /* Kill the process.  On W32 this doesn't kill child processes
+	 so it doesn't work very well for shells which is why it's not
+	 used in every case.  */
+      else if (!TerminateProcess (h_process, 0xff))
+        {
+#if 0 /* FSF Emacs */
+	  DebPrint (("sys_kill.TerminateProcess returned %d "
+		     "for pid %lu\n", GetLastError (), pid));
+	  errno = EINVAL;
+#endif
+	  rc = 0;
+        }
+    }
+
+  if (close_process)
+    CloseHandle (h_process);
+
+  return rc;
+}
+
+/* -------------------------- all-OS functions ---------------------------- */
+
+static int
+send_signal (struct nt_process_data *cp, int pid, int signo)
+{
+  return send_signal_the_nt_way (cp, pid, signo)
+    || send_signal_the_95_way (cp, pid, signo);
+}
+
 /*
  * Signal error if SIGNO is not supported
  */
@@ -383,8 +642,8 @@ static void
 nt_finalize_process_data (Lisp_Process *p, int for_disksave)
 {
   assert (!for_disksave);
-  if (NT_DATA(p)->h_process)
-    CloseHandle (NT_DATA(p)->h_process);
+  if (NT_DATA (p)->h_process)
+    CloseHandle (NT_DATA (p)->h_process);
 }
 
 /*
@@ -416,6 +675,13 @@ signal_cannot_launch (Lisp_Object image_file, DWORD err)
   signal_simple_error_2 ("Error starting", image_file, lisp_strerror (errno));
 }
 
+static void
+ensure_console_window_exists ()
+{
+  if (msw_windows9x_p ())
+    msw_hide_console ();
+}
+
 static int
 nt_create_process (Lisp_Process *p,
 		   Lisp_Object *argv, int nargv,
@@ -595,6 +861,7 @@ nt_create_process (Lisp_Process *p,
     STARTUPINFO si;
     PROCESS_INFORMATION pi;
     DWORD err;
+    DWORD flags;
 
     xzero (si);
     si.dwFlags = STARTF_USESHOWWINDOW;
@@ -607,9 +874,19 @@ nt_create_process (Lisp_Process *p,
 	si.dwFlags |= STARTF_USESTDHANDLES;
       }
 
-    err = (CreateProcess (NULL, command_line, NULL, NULL, TRUE,
-			  CREATE_NEW_CONSOLE | CREATE_NEW_PROCESS_GROUP
-			  | CREATE_SUSPENDED,
+    flags = CREATE_SUSPENDED;
+    if (msw_windows9x_p ())
+      flags |= (!NILP (Vmswindows_start_process_share_console)
+		? CREATE_NEW_PROCESS_GROUP
+		: CREATE_NEW_CONSOLE);
+    else
+      flags |= CREATE_NEW_CONSOLE | CREATE_NEW_PROCESS_GROUP;
+    if (NILP (Vmswindows_start_process_inherit_error_mode))
+      flags |= CREATE_DEFAULT_ERROR_MODE;
+
+    ensure_console_window_exists ();
+
+    err = (CreateProcess (NULL, command_line, NULL, NULL, TRUE, flags,
 			  proc_env, (char *) XSTRING_DATA (cur_dir), &si, &pi)
 	   ? 0 : GetLastError ());
 
@@ -636,6 +913,7 @@ nt_create_process (Lisp_Process *p,
     if (do_io)
       {
 	NT_DATA(p)->h_process = pi.hProcess;
+	NT_DATA(p)->dwProcessId = pi.dwProcessId;
 	init_process_io_handles (p, (void*)hmyslurp, (void*)hmyshove, 0);
       }
     else
@@ -789,18 +1067,18 @@ nt_kill_child_process (Lisp_Object proc, int signo,
 
   /* Enable child signals if necessary.  This may lose the first
      but it's better than nothing. */
-  if (NT_DATA(p)->need_enable_child_signals > 0)
+  if (NT_DATA (p)->need_enable_child_signals > 0)
     {
-      enable_child_signals(NT_DATA(p)->h_process);
-      NT_DATA(p)->need_enable_child_signals = 0;
+      enable_child_signals (NT_DATA(p)->h_process);
+      NT_DATA (p)->need_enable_child_signals = 0;
     }
 
   /* Signal error if SIGNO cannot be sent */
   validate_signal_number (signo);
 
   /* Send signal */
-  if (!send_signal (NT_DATA(p)->h_process, signo))
-    error ("Cannot send signal to process");
+  if (!send_signal (NT_DATA (p), 0, signo))
+    signal_simple_error ("Cannot send signal to process", proc);
 }
 
 /*
@@ -812,26 +1090,13 @@ nt_kill_child_process (Lisp_Object proc, int signo,
 static int
 nt_kill_process_by_pid (int pid, int signo)
 {
-  HANDLE h_process;
-  int send_result;
-  
+  struct Lisp_Process *p;
+
   /* Signal error if SIGNO cannot be sent */
   validate_signal_number (signo);
 
-  /* Try to open the process with required privileges */
-  h_process = OpenProcess (PROCESS_CREATE_THREAD
-			   | PROCESS_QUERY_INFORMATION 
-			   | PROCESS_VM_OPERATION
-			   | PROCESS_VM_WRITE,
-			   FALSE, pid);
-  if (h_process == NULL)
-    return -1;
-  
-  send_result = send_signal (h_process, signo);
-  
-  CloseHandle (h_process);
-
-  return send_result ? 0 : -1;
+  p = find_process_from_pid (pid);
+  return send_signal (p ? NT_DATA (p) : 0, pid, signo) ? 0 : -1;
 }
 
 /*-----------------------------------------------------------------------*/
@@ -942,9 +1207,11 @@ nt_canonicalize_host_name (Lisp_Object host)
    deactivate and close it via delete-process */
 
 static void
-nt_open_network_stream (Lisp_Object name, Lisp_Object host, Lisp_Object service,
+nt_open_network_stream (Lisp_Object name, Lisp_Object host,
+			Lisp_Object service,
 			Lisp_Object protocol, void** vinfd, void** voutfd)
 {
+  /* !!#### not Mule-ized */
   struct sockaddr_in address;
   SOCKET s;
   int port;
@@ -953,8 +1220,7 @@ nt_open_network_stream (Lisp_Object name, Lisp_Object host, Lisp_Object service,
   CHECK_STRING (host);
 
   if (!EQ (protocol, Qtcp))
-    error ("Unsupported protocol \"%s\"",
-	   string_data (symbol_name (XSYMBOL (protocol))));
+    signal_simple_error ("Unsupported protocol", protocol);
 
   if (INTP (service))
     port = htons ((unsigned short) XINT (service));
@@ -964,7 +1230,7 @@ nt_open_network_stream (Lisp_Object name, Lisp_Object host, Lisp_Object service,
       CHECK_STRING (service);
       svc_info = getservbyname ((char *) XSTRING_DATA (service), "tcp");
       if (svc_info == 0)
-	error ("Unknown service \"%s\"", XSTRING_DATA (service));
+	signal_simple_error ("Unknown service", service);
       port = svc_info->s_port;
     }
 
@@ -1026,18 +1292,20 @@ nt_open_network_stream (Lisp_Object name, Lisp_Object host, Lisp_Object service,
 
  connect_failed:  
   closesocket (s);
-  if (INTP (service)) {
-    warn_when_safe(Qstream, Qwarning,
-		   "failure to open network stream to host \"%s\" for service \"%d\"",
-		   XSTRING_DATA (host),
-		   (unsigned short) XINT (service));
-  }
-  else {
-    warn_when_safe(Qstream, Qwarning,
-		   "failure to open network stream to host \"%s\" for service \"%s\"",
-		   XSTRING_DATA (host),
-		   XSTRING_DATA (service));
-  }
+  if (INTP (service))
+    {
+      warn_when_safe (Qstream, Qwarning,
+		      "failure to open network stream to host \"%s\" for service \"%d\"",
+		      XSTRING_DATA (host),
+		      (unsigned short) XINT (service));
+    }
+  else
+    {
+      warn_when_safe (Qstream, Qwarning,
+		      "failure to open network stream to host \"%s\" for service \"%s\"",
+		      XSTRING_DATA (host),
+		      XSTRING_DATA (service));
+    }
   report_file_error ("connection failed", list2 (host, name));
 }
 
@@ -1077,4 +1345,22 @@ syms_of_process_nt (void)
 void
 vars_of_process_nt (void)
 {
+  DEFVAR_LISP ("mswindows-start-process-share-console",
+	       &Vmswindows_start_process_share_console /*
+When nil, new child processes are given a new console.
+When non-nil, they share the Emacs console; this has the limitation of
+allowing only only DOS subprocess to run at a time (whether started directly
+or indirectly by Emacs), and preventing Emacs from cleanly terminating the
+subprocess group, but may allow Emacs to interrupt a subprocess that doesn't
+otherwise respond to interrupts from Emacs.
+*/ );
+  Vmswindows_start_process_share_console = Qnil;
+
+  DEFVAR_LISP ("mswindows-start-process-inherit-error-mode",
+	       &Vmswindows_start_process_inherit_error_mode /*
+    "When nil, new child processes revert to the default error mode.
+When non-nil, they inherit their error mode setting from Emacs, which stops
+them blocking when trying to access unmounted drives etc.
+*/ );
+  Vmswindows_start_process_inherit_error_mode = Qt;
 }
