@@ -18,26 +18,26 @@
  (make-extent (point) (point))
  (setq radio-button1 
        (make-glyph 
-	[button :descriptor ["ok     " (setq ok-select t)
+	[button :descriptor ["ok1" (setq ok-select t)
 			     :style radio :selected ok-select]])))
 ;; button in a group
 (set-extent-begin-glyph 
  (make-extent (point) (point))
  (setq radio-button2
        (make-glyph
-	[button :descriptor ["ok" (setq ok-select nil) :style radio 
+`	[button :descriptor ["ok2" (setq ok-select nil) :style radio 
 			     :selected (not ok-select)]])))
 ;; toggle button
 (set-extent-begin-glyph 
  (make-extent (point) (point))
  (setq tbutton
-       (make-glyph [button :descriptor ["ok" (setq ok-select nil) 
+       (make-glyph [button :descriptor ["ok3" (setq ok-select nil) 
 					:style toggle 
 					:selected (not ok-select)]])))
 (set-extent-begin-glyph 
  (make-extent (point) (point))
  (setq toggle-button
-       (make-glyph [button :descriptor ["ok" :style toggle 
+       (make-glyph [button :descriptor ["ok4" :style toggle 
 					:callback 
 					(setq ok-select (not ok-select))
 					:selected ok-select]])))
@@ -69,9 +69,9 @@
 	    [tab-control :descriptor "My Tab"
 			 :face highlight
 			 :orientation right
-			 :properties (:items (["One" foo]
-					      ["Two" fee]
-					      ["Three" foo]))])))
+			 :properties (:items (["One" foo :selected t]
+					      ["Two" fee :selected nil]
+					      ["Three" foo :selected nil]))])))
 
 ;; progress gauge
 (set-extent-begin-glyph 
